@@ -3,7 +3,7 @@
  *
  * Copyright (c) 2001 Dug Song <dugsong@monkey.org>
  *
- * $Id: fw-pf.c,v 1.12 2003/01/21 18:46:03 dugsong Exp $
+ * $Id: fw-pf.c,v 1.21 2005/03/15 05:05:37 dugsong Exp $
  */
 
 #include "config.h"
@@ -28,27 +28,40 @@
 
 /*
  * XXX - cope with moving pf API
- *     $OpenBSD: pfvar.h,v 1.130 2003/01/09 10:40:45 cedric Exp $
- *     $OpenBSD: pfvar.h,v 1.102 2002/11/23 05:16:58 mcbride Exp $
- *     $OpenBSD: pfvar.h,v 1.68 2002/04/24 18:10:25 dhartmei Exp $
  */
-#if defined(DIOCRINABEGIN)
+#if defined(DIOCRCLRTABLES)
+/* XXX - can't isolate the following change:
+ *     $OpenBSD: pfvar.h,v 1.112 2002/12/17 12:30:13 mcbride Exp $
+ *  so i'll take 1.119's DIOCRCLRTABLES - 12 days of pf unsupported.
+ */
+# define HAVE_PF_CHANGE_GET_TICKET	1
+/* OpenBSD 3.3+ - 3.6 */
+/*     $OpenBSD: pfvar.h,v 1.197 2004/06/14 20:53:27 cedric Exp $ */
+/*     $OpenBSD: pfvar.h,v 1.130 2003/01/09 10:40:45 cedric Exp $ */
+/*     $OpenBSD: pfvar.h,v 1.127 2003/01/05 22:14:23 dhartmei Exp $ */
 # define PFRA_ADDR(ra)	(ra)->addr.v.a.addr.v4.s_addr
 # define PFRA_MASK(ra)	(ra)->addr.v.a.mask.v4.s_addr
 # define pfioc_changerule	pfioc_rule
 # define oldrule	rule
 # define newrule	rule
 #elif defined(DIOCBEGINADDRS)
+/*     $OpenBSD: pfvar.h,v 1.102 2002/11/23 05:16:58 mcbride Exp $ */
 # define PFRA_ADDR(ra)	(ra)->addr.addr.v4.s_addr
 # define PFRA_MASK(ra)	(ra)->addr.mask.v4.s_addr
 #elif defined(PFRULE_FRAGMENT)
 /* OpenBSD 3.2 */
+/*     $OpenBSD: pfvar.h,v 1.68 2002/04/24 18:10:25 dhartmei Exp $ */
 # define PFRA_ADDR(ra)	(ra)->addr.addr.v4.s_addr
 # define PFRA_MASK(ra)	(ra)->mask.v4.s_addr
-#else
+#elif defined (PF_AEQ)
 /* OpenBSD 3.1 */
+/*     $OpenBSD: pfvar.h,v 1.51 2001/09/15 03:54:40 frantzen Exp $ */
 # define PFRA_ADDR(ra)	(ra)->addr.v4.s_addr
 # define PFRA_MASK(ra)	(ra)->mask.v4.s_addr
+#else
+/* OpenBSD 3.0 */
+# define PFRA_ADDR(ra)	(ra)->addr
+# define PFRA_ADDR(ra)	(ra)->mask
 #endif
 
 struct fw_handle {
@@ -86,16 +99,16 @@ fr_to_pr(const struct fw_rule *fr, struct pf_rule *pr)
 	case IP_PROTO_UDP:
 		pr->src.port[0] = htons(fr->fw_sport[0]);
 		pr->src.port[1] = htons(fr->fw_sport[1]);
-		if (pr->src.port[0] == pr->src.port[1])
+		if (pr->src.port[0] == pr->src.port[1]) {
 			pr->src.port_op = PF_OP_EQ;
-		else
+		} else
 			pr->src.port_op = PF_OP_IRG;
 
 		pr->dst.port[0] = htons(fr->fw_dport[0]);
 		pr->dst.port[1] = htons(fr->fw_dport[1]);
-		if (pr->dst.port[0] == pr->dst.port[1])
+		if (pr->dst.port[0] == pr->dst.port[1]) {
 			pr->dst.port_op = PF_OP_EQ;
-		else
+		} else
 			pr->dst.port_op = PF_OP_IRG;
 		break;
 	}
@@ -155,17 +168,30 @@ pr_to_fr(const struct pf_rule *pr, struct fw_rule *fr)
 	return (0);
 }
 
+#ifdef HAVE_PF_CHANGE_GET_TICKET
+static int
+_fw_cmp(const struct fw_rule *a, const struct fw_rule *b)
+{
+	if (strcmp(a->fw_device, b->fw_device) != 0 || a->fw_op != b->fw_op ||
+	                a->fw_dir != b->fw_dir || a->fw_proto != b->fw_proto ||
+	    addr_cmp(&a->fw_src, &b->fw_src) != 0 ||
+	    addr_cmp(&a->fw_dst, &b->fw_dst) != 0 ||
+	    memcmp(a->fw_sport, b->fw_sport, sizeof(a->fw_sport)) != 0 ||
+	    memcmp(a->fw_dport, b->fw_dport, sizeof(a->fw_dport)) != 0)
+		return (-1);
+	return (0);
+}
+#endif
+
 fw_t *
 fw_open(void)
 {
 	fw_t *fw;
 
-	if ((fw = calloc(1, sizeof(*fw))) == NULL)
-		return (NULL);
-
-	if ((fw->fd = open("/dev/pf", O_RDWR)) < 0)
-		return (fw_close(fw));
-	
+	if ((fw = calloc(1, sizeof(*fw))) != NULL) {
+		if ((fw->fd = open("/dev/pf", O_RDWR)) < 0)
+			return (fw_close(fw));
+	}
 	return (fw);
 }
 
@@ -173,11 +199,37 @@ int
 fw_add(fw_t *fw, const struct fw_rule *rule)
 {
 	struct pfioc_changerule pcr;
-	
+
 	assert(fw != NULL && rule != NULL);
 	memset(&pcr, 0, sizeof(pcr));
-	fr_to_pr(rule, &pcr.newrule);
+#ifdef HAVE_PF_CHANGE_GET_TICKET
+	{
+		struct fw_rule fr;
+		
+		if (ioctl(fw->fd, DIOCGETRULES, &pcr) < 0)
+			return (-1);
+		while ((int)--pcr.nr >= 0) {
+			if (ioctl(fw->fd, DIOCGETRULE, &pcr) == 0 &&
+			    pr_to_fr(&pcr.rule, &fr) == 0) {
+				if (_fw_cmp(rule, &fr) == 0) {
+					errno = EEXIST;
+					return (-1);
+				}
+			}
+		}
+	}
+#endif
+#ifdef DIOCBEGINADDRS
+	{
+		struct pfioc_pooladdr ppa;
+		
+		if (ioctl(fw->fd, DIOCBEGINADDRS, &ppa) < 0)
+			return (-1);
+		pcr.pool_ticket = ppa.ticket;
+	}
+#endif
 	pcr.action = PF_CHANGE_ADD_TAIL;
+	fr_to_pr(rule, &pcr.newrule);
 	
 	return (ioctl(fw->fd, DIOCCHANGERULE, &pcr));
 }
@@ -189,9 +241,40 @@ fw_delete(fw_t *fw, const struct fw_rule *rule)
 	
 	assert(fw != NULL && rule != NULL);
 	memset(&pcr, 0, sizeof(pcr));
-	fr_to_pr(rule, &pcr.oldrule);
+#ifdef HAVE_PF_CHANGE_GET_TICKET
+	{
+		struct fw_rule fr;
+		int found = 0;
+		
+		if (ioctl(fw->fd, DIOCGETRULES, &pcr) < 0)
+			return (-1);
+		while ((int)--pcr.nr >= 0) {
+			if (ioctl(fw->fd, DIOCGETRULE, &pcr) == 0 &&
+			    pr_to_fr(&pcr.rule, &fr) == 0) {
+				if (_fw_cmp(rule, &fr) == 0) {
+					found = 1;
+					break;
+				}
+			}
+		}
+		if (!found) {
+			errno = ENOENT;
+			return (-1);
+		}
+	}
+#endif
+#ifdef DIOCBEGINADDRS
+	{
+		struct pfioc_pooladdr ppa;
+		
+		if (ioctl(fw->fd, DIOCBEGINADDRS, &ppa) < 0)
+			return (-1);
+		pcr.pool_ticket = ppa.ticket;
+	}
+#endif
 	pcr.action = PF_CHANGE_REMOVE;
-
+	fr_to_pr(rule, &pcr.oldrule);
+	
 	return (ioctl(fw->fd, DIOCCHANGERULE, &pcr));
 }
 
@@ -212,6 +295,12 @@ fw_loop(fw_t *fw, fw_handler callback, void *arg)
 		
 		if ((ret = ioctl(fw->fd, DIOCGETRULE, &pr)) < 0)
 			break;
+#ifdef PF_TABLE_NAME_SIZE
+		/* XXX - actually in r1.125, not 1.126 */
+		if (pr.rule.src.addr.type == PF_ADDR_TABLE ||
+		    pr.rule.dst.addr.type == PF_ADDR_TABLE)
+			continue;
+#endif
 		if (pr_to_fr(&pr.rule, &fr) < 0)
 			continue;
 		if ((ret = callback(&fr, arg)) != 0)
@@ -223,10 +312,10 @@ fw_loop(fw_t *fw, fw_handler callback, void *arg)
 fw_t *
 fw_close(fw_t *fw)
 {
-	assert(fw != NULL);
-
-	if (fw->fd > 0)
-		close(fw->fd);
-	free(fw);
+	if (fw != NULL) {
+		if (fw->fd >= 0)
+			close(fw->fd);
+		free(fw);
+	}
 	return (NULL);
 }
