@@ -3,7 +3,7 @@
  *
  * Copyright (c) 2001 Dug Song <dugsong@monkey.org>
  *
- * $Id: intf.c,v 1.38 2002/03/29 01:40:01 dugsong Exp $
+ * $Id: intf.c,v 1.56 2006/01/09 07:09:49 dugsong Exp $
  */
 
 #include "config.h"
@@ -13,9 +13,22 @@
 #include <sys/ioctl.h>
 #include <sys/socket.h>
 #ifdef HAVE_SYS_SOCKIO_H
-#include <sys/sockio.h>
+# include <sys/sockio.h>
+#endif
+/* XXX - AIX */
+#ifndef IP_MULTICAST
+# define IP_MULTICAST
 #endif
 #include <net/if.h>
+#ifdef HAVE_NET_IF_VAR_H
+# include <net/if_var.h>
+#endif
+#undef IP_MULTICAST
+/* XXX - IPv6 ioctls */
+#ifdef HAVE_NETINET_IN_VAR_H
+# include <netinet/in.h>
+# include <netinet/in_var.h>
+#endif
 
 #include <errno.h>
 #include <stdio.h>
@@ -44,7 +57,8 @@
 
 #ifdef HAVE_SOCKADDR_SA_LEN
 # define NEXTIFR(i)	((struct ifreq *)((u_char *)&i->ifr_addr + \
-				i->ifr_addr.sa_len))
+				(i->ifr_addr.sa_len ? i->ifr_addr.sa_len : \
+				 sizeof(i->ifr_addr))))
 #else
 # define NEXTIFR(i)	(i + 1)
 #endif
@@ -60,6 +74,7 @@ struct dnet_ifaliasreq {
 
 struct intf_handle {
 	int		fd;
+	int		fd6;
 	struct ifconf	ifc;
 	u_char		ifcbuf[4192];
 };
@@ -105,15 +120,50 @@ intf_open(void)
 {
 	intf_t *intf;
 	
-	if ((intf = calloc(1, sizeof(*intf))) == NULL)
-		return (NULL);
-
-	if ((intf->fd = socket(AF_INET, SOCK_DGRAM, 0)) < 0)
-		return (intf_close(intf));
-
+	if ((intf = calloc(1, sizeof(*intf))) != NULL) {
+		intf->fd = intf->fd6 = -1;
+		
+		if ((intf->fd = socket(AF_INET, SOCK_DGRAM, 0)) < 0)
+			return (intf_close(intf));
+#ifdef SIOCGIFNETMASK_IN6
+		if ((intf->fd6 = socket(AF_INET6, SOCK_DGRAM, 0)) < 0) {
+#  ifdef EPROTONOSUPPORT
+			if (errno != EPROTONOSUPPORT)
+#  endif
+				return (intf_close(intf));
+		}
+#endif
+	}
 	return (intf);
 }
 
+static int
+_intf_delete_addrs(intf_t *intf, struct intf_entry *entry)
+{
+#if defined(SIOCDIFADDR)
+	struct dnet_ifaliasreq ifra;
+	
+	memset(&ifra, 0, sizeof(ifra));
+	strlcpy(ifra.ifra_name, entry->intf_name, sizeof(ifra.ifra_name));
+	if (entry->intf_addr.addr_type == ADDR_TYPE_IP) {
+		addr_ntos(&entry->intf_addr, &ifra.ifra_addr);
+		ioctl(intf->fd, SIOCDIFADDR, &ifra);
+	}
+	if (entry->intf_dst_addr.addr_type == ADDR_TYPE_IP) {
+		addr_ntos(&entry->intf_dst_addr, &ifra.ifra_addr);
+		ioctl(intf->fd, SIOCDIFADDR, &ifra);
+	}
+#elif defined(SIOCLIFREMOVEIF)
+	struct ifreq ifr;
+
+	memset(&ifr, 0, sizeof(ifr));
+	strlcpy(ifr.ifr_name, entry->intf_name, sizeof(ifr.ifr_name));
+	/* XXX - overloading Solaris lifreq with ifreq */
+	ioctl(intf->fd, SIOCLIFREMOVEIF, &ifr);
+#endif
+	return (0);
+}
+
 static int
 _intf_delete_aliases(intf_t *intf, struct intf_entry *entry)
 {
@@ -124,7 +174,7 @@ _intf_delete_aliases(intf_t *intf, struct intf_entry *entry)
 	memset(&ifra, 0, sizeof(ifra));
 	strlcpy(ifra.ifra_name, entry->intf_name, sizeof(ifra.ifra_name));
 	
-	for (i = 0; i < entry->intf_alias_num; i++) {
+	for (i = 0; i < (int)entry->intf_alias_num; i++) {
 		addr_ntos(&entry->intf_alias_addrs[i], &ifra.ifra_addr);
 		ioctl(intf->fd, SIOCDIFADDR, &ifra);
 	}
@@ -158,7 +208,10 @@ _intf_add_aliases(intf_t *intf, const struct intf_entry *entry)
 	memset(&ifra, 0, sizeof(ifra));
 	strlcpy(ifra.ifra_name, entry->intf_name, sizeof(ifra.ifra_name));
 	
-	for (i = 0; i < entry->intf_alias_num; i++) {
+	for (i = 0; i < (int)entry->intf_alias_num; i++) {
+		if (entry->intf_alias_addrs[i].addr_type != ADDR_TYPE_IP)
+			continue;
+		
 		if (addr_ntos(&entry->intf_alias_addrs[i],
 		    &ifra.ifra_addr) < 0)
 			return (-1);
@@ -172,10 +225,14 @@ _intf_add_aliases(intf_t *intf, const struct intf_entry *entry)
 	}
 #else
 	struct ifreq ifr;
+	int n = 1;
 	
 	for (i = 0; i < entry->intf_alias_num; i++) {
+		if (entry->intf_alias_addrs[i].addr_type != ADDR_TYPE_IP)
+			continue;
+		
 		snprintf(ifr.ifr_name, sizeof(ifr.ifr_name), "%s:%d",
-		    entry->intf_name, i + 1);
+		    entry->intf_name, n++);
 # ifdef SIOCLIFADDIF
 		if (ioctl(intf->fd, SIOCLIFADDIF, &ifr) < 0)
 			return (-1);
@@ -204,7 +261,15 @@ intf_set(intf_t *intf, const struct intf_entry *entry)
 	
 	if (intf_get(intf, orig) < 0)
 		return (-1);
+	
+	/* Delete any existing aliases. */
+	if (_intf_delete_aliases(intf, orig) < 0)
+		return (-1);
 
+	/* Delete any existing addrs. */
+	if (_intf_delete_addrs(intf, orig) < 0)
+		return (-1);
+	
 	memset(&ifr, 0, sizeof(ifr));
 	strlcpy(ifr.ifr_name, entry->intf_name, sizeof(ifr.ifr_name));
 	
@@ -229,8 +294,11 @@ intf_set(intf_t *intf, const struct intf_entry *entry)
 		if (ioctl(intf->fd, SIOCSIFADDR, &ifr) < 0 && errno != EEXIST)
 			return (-1);
 		
-		if (addr_btos(entry->intf_addr.addr_bits,
-		    &ifr.ifr_addr) == 0) {
+		if (addr_btos(entry->intf_addr.addr_bits, &ifr.ifr_addr) == 0
+#ifdef __linux__
+		    && entry->intf_addr.addr_ip != 0
+#endif
+		    ) {
 			if (ioctl(intf->fd, SIOCSIFNETMASK, &ifr) < 0)
 				return (-1);
 		}
@@ -275,10 +343,6 @@ intf_set(intf_t *intf, const struct intf_entry *entry)
 		    errno != EEXIST)
 			return (-1);
 	}
-	/* Delete any existing aliases. */
-	if (_intf_delete_aliases(intf, orig) < 0)
-		return (-1);
-	
 	/* Add aliases. */
 	if (_intf_add_aliases(intf, entry) < 0)
 		return (-1);
@@ -351,6 +415,14 @@ _intf_get_noalias(intf_t *intf, struct intf_entry *entry)
 			return (-1);
 		if (addr_ston(&ifr.ifr_addr, &entry->intf_link_addr) < 0)
 			return (-1);
+#elif defined(SIOCRPHYSADDR)
+		/* Tru64 */
+		struct ifdevea *ifd = (struct ifdevea *)&ifr; /* XXX */
+		
+		if (ioctl(intf->fd, SIOCRPHYSADDR, ifd) < 0)
+			return (-1);
+		addr_pack(&entry->intf_link_addr, ADDR_TYPE_ETH, ETH_ADDR_BITS,
+		    ifd->current_pa, ETH_ADDR_LEN);
 #else
 		eth_t *eth;
 		
@@ -386,7 +458,7 @@ _intf_get_aliases(intf_t *intf, struct intf_entry *entry)
 	lap = (struct addr *)((u_char *)entry + entry->intf_len);
 	
 	while (ioctl(intf->fd, SIOCLIFADDR, &ifra) == 0 &&
-	    ifra.ifra_cookie > 0 && ap < lap) {
+	    ifra.ifra_cookie > 0 && (ap + 1) < lap) {
 		if (addr_ston(&ifra.ifra_addr, ap) < 0)
 			break;
 		ap++, entry->intf_alias_num++;
@@ -403,17 +475,18 @@ _intf_get_aliases(intf_t *intf, struct intf_entry *entry)
 	struct addr *ap, *lap;
 	char *p;
 	
-	if (intf->ifc.ifc_len < sizeof(*ifr)) {
+	if (intf->ifc.ifc_len < (int)sizeof(*ifr)) {
 		errno = EINVAL;
 		return (-1);
 	}
 	entry->intf_alias_num = 0;
 	ap = entry->intf_alias_addrs;
-	lifr = (struct ifreq *)&intf->ifc.ifc_buf[intf->ifc.ifc_len];
+	lifr = (struct ifreq *)intf->ifc.ifc_buf + 
+	    (intf->ifc.ifc_len / sizeof(*lifr));
 	lap = (struct addr *)((u_char *)entry + entry->intf_len);
 	
 	/* Get addresses for this interface. */
-	for (ifr = intf->ifc.ifc_req; ifr < lifr && ap < lap;
+	for (ifr = intf->ifc.ifc_req; ifr < lifr && (ap + 1) < lap;
 	    ifr = NEXTIFR(ifr)) {
 		/* XXX - Linux, Solaris ifaliases */
 		if ((p = strchr(ifr->ifr_name, ':')) != NULL)
@@ -422,16 +495,58 @@ _intf_get_aliases(intf_t *intf, struct intf_entry *entry)
 		if (strcmp(ifr->ifr_name, entry->intf_name) != 0)
 			continue;
 		
-		if (addr_ston(&ifr->ifr_addr, ap) < 0 ||
-		    ap->addr_type != ADDR_TYPE_IP)
+		if (addr_ston(&ifr->ifr_addr, ap) < 0)
 			continue;
 		
-		if (ap->addr_ip == entry->intf_addr.addr_ip ||
-		    ap->addr_ip == entry->intf_dst_addr.addr_ip)
+		/* XXX */
+		if (ap->addr_type == ADDR_TYPE_ETH) {
+			memcpy(&entry->intf_link_addr, ap, sizeof(*ap));
 			continue;
-		
+		} else if (ap->addr_type == ADDR_TYPE_IP) {
+			if (ap->addr_ip == entry->intf_addr.addr_ip ||
+			    ap->addr_ip == entry->intf_dst_addr.addr_ip)
+				continue;
+		}
+#ifdef SIOCGIFNETMASK_IN6
+		else if (ap->addr_type == ADDR_TYPE_IP6 && intf->fd6 != -1) {
+			struct in6_ifreq ifr6;
+
+			/* XXX - sizeof(ifr) < sizeof(ifr6) */
+			memcpy(&ifr6, ifr, sizeof(ifr6));
+			
+			if (ioctl(intf->fd6, SIOCGIFNETMASK_IN6, &ifr6) == 0) {
+				addr_stob((struct sockaddr *)&ifr6.ifr_addr,
+				    &ap->addr_bits);
+			}
+			else perror("SIOCGIFNETMASK_IN6");
+		}
+#endif
 		ap++, entry->intf_alias_num++;
 	}
+#ifdef HAVE_LINUX_PROCFS
+#define PROC_INET6_FILE	"/proc/net/if_inet6"
+	{
+		FILE *f;
+		char buf[256], s[8][5], name[INTF_NAME_LEN];
+		u_int idx, bits, scope, flags;
+		
+		if ((f = fopen(PROC_INET6_FILE, "r")) != NULL) {
+			while (ap < lap &&
+			       fgets(buf, sizeof(buf), f) != NULL) {
+				sscanf(buf, "%04s%04s%04s%04s%04s%04s%04s%04s %02x %02x %02x %02x %32s\n",
+				    s[0], s[1], s[2], s[3], s[4], s[5], s[6], s[7],
+				    &idx, &bits, &scope, &flags, name);
+				if (strcmp(name, entry->intf_name) == 0) {
+					snprintf(buf, sizeof(buf), "%s:%s:%s:%s:%s:%s:%s:%s/%d",
+					    s[0], s[1], s[2], s[3], s[4], s[5], s[6], s[7], bits);
+					addr_aton(buf, ap);
+					ap++, entry->intf_alias_num++;
+				}
+			}
+			fclose(f);
+		}
+	}
+#endif
 	entry->intf_len = (u_char *)ap - (u_char *)entry;
 	
 	return (0);
@@ -459,19 +574,21 @@ _match_intf_src(const struct intf_entry *entry, void *arg)
 	struct intf_entry *save = (struct intf_entry *)arg;
 	
 	if (entry->intf_addr.addr_type == ADDR_TYPE_IP &&
-	    entry->intf_addr.addr_ip == save->intf_addr.addr_ip &&
-	    entry->intf_len <= save->intf_len) {
-		save->intf_len = entry->intf_len;
-		memcpy(save, entry, entry->intf_len);
+	    entry->intf_addr.addr_ip == save->intf_addr.addr_ip) {
+		/* XXX - truncated result if entry is too small. */
+		if (save->intf_len < entry->intf_len)
+			memcpy(save, entry, save->intf_len);
+		else
+			memcpy(save, entry, entry->intf_len);
 		return (1);
 	}
 	return (0);
 }
 
 int
-intf_get_src(intf_t *intf, struct intf_entry *entry, struct addr *dst)
+intf_get_src(intf_t *intf, struct intf_entry *entry, struct addr *src)
 {
-	memcpy(&entry->intf_addr, dst, sizeof(*dst));
+	memcpy(&entry->intf_addr, src, sizeof(*src));
 	
 	if (intf_loop(intf, _match_intf_src, entry) != 1) {
 		errno = ENXIO;
@@ -486,16 +603,20 @@ intf_get_dst(intf_t *intf, struct intf_entry *entry, struct addr *dst)
 	struct sockaddr_in sin;
 	int n;
 	
+	if (dst->addr_type != ADDR_TYPE_IP) {
+		errno = EINVAL;
+		return (-1);
+	}
 	addr_ntos(dst, (struct sockaddr *)&sin);
 	sin.sin_port = htons(666);
-
+	
 	if (connect(intf->fd, (struct sockaddr *)&sin, sizeof(sin)) < 0)
 		return (-1);
-
+	
 	n = sizeof(sin);
 	if (getsockname(intf->fd, (struct sockaddr *)&sin, &n) < 0)
 		return (-1);
-
+	
 	addr_ston((struct sockaddr *)&sin, &entry->intf_addr);
 	
 	if (intf_loop(intf, _match_intf_src, entry) != 1)
@@ -533,7 +654,8 @@ intf_loop(intf_t *intf, intf_handler callback, void *arg)
 		*p = '\0';
 		for (p = buf; *p == ' '; p++)
 			;
-		
+
+		memset(ebuf, 0, sizeof(ebuf));
 		strlcpy(entry->intf_name, p, sizeof(entry->intf_name));
 		entry->intf_len = sizeof(ebuf);
 		
@@ -583,6 +705,7 @@ intf_loop(intf_t *intf, intf_handler callback, void *arg)
 		if (pifr != NULL && strcmp(ifr->ifr_name, pifr->ifr_name) == 0)
 			continue;
 
+		memset(ebuf, 0, sizeof(ebuf));
 		strlcpy(entry->intf_name, ifr->ifr_name,
 		    sizeof(entry->intf_name));
 		entry->intf_len = sizeof(ebuf);
@@ -604,8 +727,12 @@ intf_loop(intf_t *intf, intf_handler callback, void *arg)
 intf_t *
 intf_close(intf_t *intf)
 {
-	if (intf->fd > 0)
-		close(intf->fd);
-	free(intf);
+	if (intf != NULL) {
+		if (intf->fd >= 0)
+			close(intf->fd);
+		if (intf->fd6 >= 0)
+			close(intf->fd6);
+		free(intf);
+	}
 	return (NULL);
 }
