@@ -3,25 +3,26 @@
  *
  * Copyright (c) 2001 Dug Song <dugsong@monkey.org>
  *
- * $Id: ip-cooked.c,v 1.12 2002/04/05 03:08:29 dugsong Exp $
+ * $Id: ip-cooked.c,v 1.17 2005/01/25 21:30:40 dugsong Exp $
  */
 
 #include "config.h"
 
+#ifndef _WIN32
 #include <netinet/in.h>
-
+#include <unistd.h>
+#endif
 #include <errno.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#include <unistd.h>
 
 #include "dnet.h"
 #include "queue.h"
 
 struct ip_intf {
 	eth_t			*eth;
-	char			 name[60];
+	char			 name[INTF_NAME_LEN];
 	struct addr		 ha;
 	struct addr		 pa;
 	int			 mtu;
@@ -68,26 +69,26 @@ ip_open(void)
 {
 	ip_t *ip;
 
-	if ((ip = calloc(1, sizeof(*ip))) == NULL)
-		return (NULL);
+	if ((ip = calloc(1, sizeof(*ip))) != NULL) {
+		ip->fd = -1;
+		
+		if ((ip->arp = arp_open()) == NULL ||
+		    (ip->intf = intf_open()) == NULL ||
+		    (ip->route = route_open()) == NULL)
+			return (ip_close(ip));
+		
+		if ((ip->fd = socket(AF_INET, SOCK_DGRAM, 0)) < 0)
+			return (ip_close(ip));
 
-	if ((ip->arp = arp_open()) == NULL ||
-	    (ip->intf = intf_open()) == NULL ||
-	    (ip->route = route_open()) == NULL)
-		return (ip_close(ip));
+		memset(&ip->sin, 0, sizeof(ip->sin));
+		ip->sin.sin_family = AF_INET;
+		ip->sin.sin_port = htons(666);
+		
+		LIST_INIT(&ip->ip_intf_list);
 
-	if ((ip->fd = socket(AF_INET, SOCK_DGRAM, 0)) < 0)
-		return (ip_close(ip));
-
-	memset(&ip->sin, 0, sizeof(ip->sin));
-	ip->sin.sin_family = AF_INET;
-	ip->sin.sin_port = htons(666);
-	
-	LIST_INIT(&ip->ip_intf_list);
-
-	if (intf_loop(ip->intf, _add_ip_intf, ip) != 0)
-		return (ip_close(ip));
-	
+		if (intf_loop(ip->intf, _add_ip_intf, ip) != 0)
+			return (ip_close(ip));
+	}
 	return (ip);
 }
 
@@ -136,7 +137,7 @@ _request_arp(struct ip_intf *ipi, struct addr *dst)
 	eth_send(ipi->eth, frame, sizeof(frame));
 }
 
-size_t
+ssize_t
 ip_send(ip_t *ip, const void *buf, size_t len)
 {
 	struct ip_hdr *iph;
@@ -222,19 +223,24 @@ ip_t *
 ip_close(ip_t *ip)
 {
 	struct ip_intf *ipi, *nxt;
-	
-	for (ipi = LIST_FIRST(&ip->ip_intf_list);
-	    ipi != LIST_END(&ip->ip_intf_list); ipi = nxt) {
-		nxt = LIST_NEXT(ipi, next);
-		if (ipi->eth != NULL)
-			eth_close(ipi->eth);
-		free(ipi);
+
+	if (ip != NULL) {
+		for (ipi = LIST_FIRST(&ip->ip_intf_list);
+		    ipi != LIST_END(&ip->ip_intf_list); ipi = nxt) {
+			nxt = LIST_NEXT(ipi, next);
+			if (ipi->eth != NULL)
+				eth_close(ipi->eth);
+			free(ipi);
+		}
+		if (ip->fd >= 0)
+			close(ip->fd);
+		if (ip->route != NULL)
+			route_close(ip->route);
+		if (ip->intf != NULL)
+			intf_close(ip->intf);
+		if (ip->arp != NULL)
+			arp_close(ip->arp);
+		free(ip);
 	}
-	if (ip->fd > 0)		close(ip->fd);
-	if (ip->route != NULL)	route_close(ip->route);
-	if (ip->intf != NULL)	intf_close(ip->intf);
-	if (ip->arp != NULL)	arp_close(ip->arp);
-	free(ip);
-	
 	return (NULL);
 }
