@@ -3,7 +3,7 @@
  *
  * Copyright (c) 2001 Dug Song <dugsong@monkey.org>
  *
- * $Id: tcp.c,v 1.5 2002/01/07 02:27:05 dugsong Exp $
+ * $Id: tcp.c,v 1.8 2002/02/08 07:43:53 dugsong Exp $
  */
 
 #include "config.h"
@@ -18,78 +18,15 @@
 #include <unistd.h>
 
 #include "dnet.h"
-#include "dnet-int.h"
+#include "aton.h"
+#include "mod.h"
 
 void
-tcp_usage(int die)
+tcp_usage(void)
 {
-	fprintf(stderr, "Usage: dnet tcp [sport|dport|flags|seq|ack|win|urp value] ...\n");
-	if (die)
-		exit(1);
-}
-
-static int
-port_aton(char *string, uint16_t *port)
-{
-	struct servent *sp;
-	long l;
-	char *p;
-	
-	if ((sp = getservbyname(string, "tcp")) != NULL) {
-		*port = sp->s_port;
-	} else {
-		l = strtol(string, &p, 10);
-		if (*string == '\0' || *p != '\0' || l > 0xffff)
-			return (-1);
-		*port = htons(l & 0xffff);
-	}
-	return (0);
-}
-
-static int
-seq_aton(char *string, uint32_t *seq)
-{
-	char *p;
-	
-	*seq = strtol(string, &p, 10);
-	if (*string == '\0' || *p != '\0')
-		return (-1);
-
-	return (0);
-}
-
-static int
-flags_aton(char *string, uint8_t *flags)
-{
-	char *p;
-
-	*flags = 0;
-	
-	for (p = string; *p != '\0'; p++) {
-		switch (*p) {
-		case 'S':
-			*flags |= TH_SYN;
-			break;
-		case 'A':
-			*flags |= TH_ACK;
-			break;
-		case 'F':
-			*flags |= TH_FIN;
-			break;
-		case 'R':
-			*flags |= TH_RST;
-			break;
-		case 'P':
-			*flags |= TH_PUSH;
-			break;
-		case 'U':
-			*flags |= TH_URG;
-			break;
-		default:
-			return (-1);
-		}
-	}
-	return (0);
+	fprintf(stderr, "Usage: dnet tcp [sport|dport|flags|seq|ack|"
+	    "win|urp <value>] ...\n");
+	exit(1);
 }
 
 int
@@ -114,39 +51,39 @@ tcp_main(int argc, char *argv[])
 	tcp->th_win = TCP_WIN_MAX;
 	tcp->th_urp = 0;
 
-	for (c = 0; c + 1 < argc; c += 2) {
+	for (c = 1; c + 1 < argc; c += 2) {
 		name = argv[c];
 		value = argv[c + 1];
 		
 		if (strcmp(name, "sport") == 0) {
 			if (port_aton(value, &tcp->th_sport) < 0)
-				tcp_usage(1);
+				tcp_usage();
 		} else if (strcmp(name, "dport") == 0) {
 			if (port_aton(value, &tcp->th_dport) < 0)
-				tcp_usage(1);
+				tcp_usage();
 		} else if (strcmp(name, "flags") == 0) {
 			if (flags_aton(value, &tcp->th_flags) < 0)
-				tcp_usage(1);
+				tcp_usage();
 		} else if (strcmp(name, "seq") == 0) {
 			if (seq_aton(value, &tcp->th_seq) < 0)
-				tcp_usage(1);
+				tcp_usage();
 		} else if (strcmp(name, "ack") == 0) {
 			if (seq_aton(value, &tcp->th_ack) < 0)
-				tcp_usage(1);
+				tcp_usage();
 		} else if (strcmp(name, "win") == 0) {
 			if (port_aton(value, &tcp->th_win) < 0)
-				tcp_usage(1);
+				tcp_usage();
 		} else if (strcmp(name, "urp") == 0) {
 			if (port_aton(value, &tcp->th_urp) < 0)
-				tcp_usage(1);
+				tcp_usage();
 		} else
-			tcp_usage(1);
+			tcp_usage();
 	}
 	argc -= c;
 	argv += c;
 	
 	if (argc != 0)
-		tcp_usage(1);
+		tcp_usage();
 
 	p = buf + TCP_HDR_LEN;
 	
@@ -164,3 +101,9 @@ tcp_main(int argc, char *argv[])
 
 	return (0);
 }
+
+struct mod mod_tcp = {
+	"tcp",
+	MOD_TYPE_ENCAP,
+	&tcp_main
+};
