@@ -3,7 +3,7 @@
  *
  * Copyright (c) 2001 Dug Song <dugsong@monkey.org>
  *
- * $Id: udp.c,v 1.5 2002/01/07 02:27:05 dugsong Exp $
+ * $Id: udp.c,v 1.8 2002/02/08 07:43:53 dugsong Exp $
  */
 
 #include "config.h"
@@ -18,32 +18,14 @@
 #include <unistd.h>
 
 #include "dnet.h"
-#include "dnet-int.h"
+#include "aton.h"
+#include "mod.h"
 
 void
-udp_usage(int die)
+udp_usage(void)
 {
-	fprintf(stderr, "Usage: dnet udp [sport|dport value] ...\n");
-	if (die)
-		exit(1);
-}
-
-static int
-port_aton(char *string, uint16_t *port)
-{
-	struct servent *sp;
-	long l;
-	char *p;
-	
-	if ((sp = getservbyname(string, "udp")) != NULL) {
-		*port = sp->s_port;
-	} else {
-		l = strtol(string, &p, 10);
-		if (*string == '\0' || *p != '\0' || l > 0xffff)
-			return (-1);
-		*port = htons(l & 0xffff);
-	}
-	return (0);
+	fprintf(stderr, "Usage: dnet udp [sport|dport <value>] ...\n");
+	exit(1);
 }
 
 int
@@ -62,24 +44,24 @@ udp_main(int argc, char *argv[])
 	udp->uh_sport = rand() & 0xffff;
 	udp->uh_dport = rand() & 0xffff;
 	
-	for (c = 0; c + 1 < argc; c += 2) {
+	for (c = 1; c + 1 < argc; c += 2) {
 		name = argv[c];
 		value = argv[c + 1];
 
 		if (strcmp(name, "sport") == 0) {
 			if (port_aton(value, &udp->uh_sport) < 0)
-				udp_usage(1);
+				udp_usage();
 		} else if (strcmp(name, "dport") == 0) {
 			if (port_aton(value, &udp->uh_dport) < 0)
-				udp_usage(1);
+				udp_usage();
 		} else
-			udp_usage(1);
+			udp_usage();
 	}
 	argc -= c;
 	argv += c;
 
 	if (argc != 0)
-		udp_usage(1);
+		udp_usage();
 
 	p = buf + UDP_HDR_LEN;
 	
@@ -98,3 +80,9 @@ udp_main(int argc, char *argv[])
 
 	return (0);
 }
+
+struct mod mod_udp = {
+	"udp",
+	MOD_TYPE_ENCAP,
+	udp_main
+};
